#include <windows.h>
#include <setupapi.h>
#include <devguid.h>
#include <regstr.h>
#include <tchar.h>
// Include WinNT.h for PHYSICAL_ADDRESS definition
#include <ntdef.h>
#include <initguid.h>
#include <ntddser.h> // Include the header for serial device definitions
#include <iostream>
#include <vector>

#pragma comment(lib, "setupapi.lib")

// Define GUID_DEVINTERFACE_COMPORT if not already defined
//#ifndef GUID_DEVINTERFACE_COMPORT
//DEFINE_GUID(GUID_DEVINTERFACE_COMPORT, 0x86E0D1E0L, 0x8089, 0x11D0, 0x9C, 0xE4, 0x08, 0x00, 0x3E, 0x30, 0x1F, 0x73);
//#endif

struct UsbDevice {
    std::wstring devicePath;
    std::wstring portName;
};

std::vector<UsbDevice> ListUsbDevices() {
    std::vector<UsbDevice> devices;

    HDEVINFO deviceInfoSet = SetupDiGetClassDevs(
        &GUID_DEVINTERFACE_COMPORT,
        NULL,
        NULL,
        DIGCF_PRESENT | DIGCF_DEVICEINTERFACE
    );

    if (deviceInfoSet == INVALID_HANDLE_VALUE) {
        return devices;
    }

    SP_DEVICE_INTERFACE_DATA deviceInterfaceData;
    deviceInterfaceData.cbSize = sizeof(SP_DEVICE_INTERFACE_DATA);
    SP_DEVINFO_DATA deviceInfoData;
    deviceInfoData.cbSize = sizeof(SP_DEVINFO_DATA);

    for (DWORD i = 0; SetupDiEnumDeviceInterfaces(deviceInfoSet, NULL, &GUID_DEVINTERFACE_COMPORT, i, &deviceInterfaceData); i++) {
        DWORD requiredSize = 0;
        SetupDiGetDeviceInterfaceDetail(deviceInfoSet, &deviceInterfaceData, NULL, 0, &requiredSize, NULL);

        PSP_DEVICE_INTERFACE_DETAIL_DATA deviceInterfaceDetailData = (PSP_DEVICE_INTERFACE_DETAIL_DATA)malloc(requiredSize);
        deviceInterfaceDetailData->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);
        if (SetupDiGetDeviceInterfaceDetail(deviceInfoSet, &deviceInterfaceData, deviceInterfaceDetailData, requiredSize, NULL, &deviceInfoData)) {
            HKEY hDeviceRegistryKey = SetupDiOpenDevRegKey(
                deviceInfoSet,
                &deviceInfoData,
                DICS_FLAG_GLOBAL,
                0,
                DIREG_DEV,
                KEY_READ
            );

            if (hDeviceRegistryKey != INVALID_HANDLE_VALUE) {
                TCHAR portName[256];
                DWORD portNameSize = sizeof(portName);
                DWORD type = 0;

                if (RegQueryValueEx(hDeviceRegistryKey, TEXT("PortName"), NULL, &type, (LPBYTE)portName, &portNameSize) == ERROR_SUCCESS) {
                    UsbDevice device;
                    device.devicePath = deviceInterfaceDetailData->DevicePath;
                    device.portName = portName;
                    devices.push_back(device);
                }

                RegCloseKey(hDeviceRegistryKey);
            }
        }

        free(deviceInterfaceDetailData);
    }

    SetupDiDestroyDeviceInfoList(deviceInfoSet);
    return devices;
}

int main() {
    std::vector<UsbDevice> devices = ListUsbDevices();

    for (const auto& device : devices) {
        std::wcout << L"Device Path: " << device.devicePath << L", Port Name: " << device.portName << std::endl;
    }

    return 0;
}
